# **<span style="color:#122d98">Visa Checkout SDK v6.6.1</span>**

To operate effectively, Visa Checkout SDK supports the following technologies:

  * Android 4.4 KitKat (API Level 19) or later
  * Android Studio 2.3 or later
  * Gradle Plugin 2.2.3 or later

## <span style="color:#122d98">Native Applications</span>

The Visa Checkout SDK for Android enables you to integrate a Visa Checkout payment flow into your Android app. The consumer taps the Visa Checkout button in your app, which launches the Visa Checkout flow. The consumer then logs into Visa Checkout to confirm the payment information and shipping address and continue with the purchase, possibly changing shipping or payment information before continuing back to your app. At that point, your app confirms the successful purchase with Visa Checkout.

After following the integration steps described in this document, your customer can checkout and pay with Visa Checkout:

![Visa Checkout mobile flow](img/flow.png)

When the customer taps the Visa Checkout button, Visa Checkout executes the following flow:

1. Visa Checkout displays the Sign In screen, which allows your customer to:
    * Sign in by entering an e-mail address and a password.
    * Create a Visa Checkout account and specify payment information and shipping details to be securely stored in Visa Checkout.
2. Visa Checkout presents the Review and Continue screen. <br/> **Note:** At this point, the customer can add or change the payment method, billing address, and shipping address.
3. After the customer taps **Continue**, Visa Checkout returns the call ID, which identifies the payment request, an encrypted payload and other information, then returns control to your app.

After control is returned to your app, you pass the encrypted payment payload to your server for authorization and capture. Once done, your server must update the payment information in Visa Checkout.

## <span style="color:#122d98">Hybrid Applications</span>

Visa Checkout provides a plugin that enables biometric authentication for your hybrid mobile app. With the plugin, consumers can use Visa Checkout with minimal friction during checkout.
The Visa Checkout plugin provides consumers a streamlined, two-touch secure checkout experience, when you integrate the plugin with your hybrid mobile app. When **Enable Fingerprint** and **Remember me** are checked, an existing consumer can use **Fingerprint** to log into Visa Checkout:

![Visa Checkout hybrid flow](img/Fingerprint_Flow.png)

A new consumer can check **Enable FingerPrint** and **Remember me** when enrolling in Visa Checkout. The consumer can choose to cancel **Fingerprint** and enter a password at any time. A consumer also can uncheck **Enable Fingerprint** to disable it without disabling **Remember me**; however, both must be checked to enable **Fingerprint**.

#### Technical Notes About the Android Hybrid App Plugin

The plugin presents the Visa Checkout flow as an Activity. When the consumer taps the Visa Checkout button, your app invokes the plugin.

- The plugin invokes the JavaScript SDK, which integrates with your website. Your existing JavaScript integration with Visa Checkout does not require modification.
- The hybrid integration consists of including the plugin in your app and initializing it with message handlers, which requires minimal coding.
- The size of the plugin is less than 200KB.

## <span style="color:#122d98">Getting Started</span>
To integrate the Android Visa Checkout SDK, review the [Installation](#installation) guide below. Then, refer to either the [Native Apps](#native_usage) or [Hybrid Apps](#hybrid_usage) usage guide.

## <span style="color:#122d98">What's new in this version</span>
- Provided Button click callback when the VisaCheckout button is clicked that helps to validate the UI if needed before launching VisaCheckout.
- VisaCheckout Button shows a spinner when it's launched if there is any delay.
- minSdkVersion support is added from API level 16. So client apps supporting minSdkVersion 16 can include VisaCheckoutSDK without any compilation errors. But Visa Checkout functionality is supported for devices running on KITKAT(API Level 19) or above. Users running below KITKAT, will not be able to use VisaCheckout functionality even it's included in the client apps.
- Redesigned look
- Prefill enrollment information for user upon user consent
- API changes

## <span style="color:#122d98">Upgrading</span>

- What's new in 6.6.1
  -  Fixed issue related to Finger print not working for returning users
  -  Memory leak issue fixed for custom button implementation
- What's new in 6.6.0 - Provided callback when the VisaCheckout button is clicked and bug fixes
- What's new in 6.3.1 - Removed READ_PHONE_STATE permission from the VisaCheckout SDK
- What's new in 6.3.0 - Support for users staying remembered when upgrading from 5.5
- What's new in 6.2.0 - Redesigned look - Prefill enrollment information for user upon user consent - API changes

### <span style="color:#122d98">Installing 6.6.1 for the first time</span>
- Review the [Installation](#installation) guide below for detailed instructions. Then, refer to either the [Native Apps](#native_usage) or [Hybrid Apps](#hybrid_usage) usage guide.


### <span style="color:#122d98">Native apps upgrading to 6.x from a previous version</span>
- Delete the old VisaCheckout Android SDK `aar` file from your libs folder.
- Add the new VisaCheckout SDK `aar` file to your libs folder.
- In your build.gradle update the name of the Visa Checkout SDK dependency to reflect the appropriate version.
- Remove any existing `VisaCheckoutSdk.init()` call.
- If you're using an instance of `VisaCheckoutButton`, use the new `CheckoutButton` class instead, and use our new `init` method that takes activity, profile, purchaseInfo and resultListener.
- If you're manually invoking Visa Checkout, use our new `VisaCheckoutSDK.initManualCheckoutSession` method instead.
- You can remove any old dependencies the previous Visa Checkout SDK libraries needed (e.g. OkHttp, multidex, gson) unless your application needs them of course.

### <span style="color:#122d98">Hybrid apps upgrading to 6.x from a previous version</span>
- Replace the VisaCheckoutHybrid.aar with visacheckout-android-sdk-6.6.1.aar.
- In your build.gradle update the name of the Visa Checkout SDK dependency to reflect the appropriate version.
- Change any `import com.visa.checkout.hybrid.VisaCheckoutPlugin;` imports to `import com.visa.checkout.VisaCheckoutHybridPlugin;` and use the `VisaCheckoutHybridPlugin` class instead of the deprecated `VisaCheckoutPlugin` class.

<a name="installation"></a>
# **<span style="color:#122d98">Installation</span>**

### <span style="color:#122d98">Add Visa Checkout library</span>

The following steps configure and set up the Visa Checkout mobile SDK for your Android Studio project.

1. Add `visacheckout-android-sdk–6.6.1.aar` file to your application's `libs` folder:  
 ![libs folder](img/libs_folder.jpg)

1. In your app module's `build.gradle` file:
 * Add the location of the SDK to the `repositories` block.
   ```
   android {
       ...
       repositories {
           ...
           flatDir { dirs 'libs' }
       }
   }
   ```
 * Add the Visa Checkout SDK to the `dependencies` block.
   ```
   dependencies {
         ...
         implementation(name:'visacheckout-android-sdk-6.6.1', ext:'aar')
   }
   ```
 * Ensure a minimum SDK version of 19 in the `defaultConfig` block.
   ```
    defaultConfig {
         ...
         minSdkVersion 19
    }
    ```

1. In your `AndroidManifest.xml` file.
  * Add the following permissions.
  ```
  <uses-permission android:name="android.permission.INTERNET"/>
  <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE"/>
  <uses-permission android:name="android.permission.USE_FINGERPRINT"/>
  ```
  * (Optional) Override `allowBackup` if it's set to true by adding the `replace` attribute.
  ```
  <application
        android:allowBackup="true"
        tools:replace="allowBackup"
        ...
  </application>
  ```

<a name="native_usage"></a>
# **<span style="color:#122d98">Usage - Native Apps</span>**

## <span style="color:#122d98">Set Up `CheckoutButton`</span>

The recommended way to use the `CheckoutButton` is to add the Visa Checkout Button in your layout and prepare the button with profile and purchase information using the `init` method. This method will provide the button with the needed information to configure Visa Checkout and prepare for a button tap.

In your layout add the Visa Checkout Button in the following way:

```xml
 <com.visa.checkout.CheckoutButton
    android:id="@+id/btn_visa_checkout"
    android:layout_width="@dimen/button_width"
    android:layout_height="@dimen/button_height"/>
```

*The following code snippet sets the profile and purchase information for the button.*

```Java
Profile profile = new Profile.ProfileBuilder("abc123...", Environment.SANDBOX)
                .setProfileName("default")
                .build();

PurchaseInfo purchaseInfo = new PurchaseInfo.PurchaseInfoBuilder(new BigDecimal("10.23"),
                PurchaseInfo.Currency.USD)
                .build();

visaCheckoutButton = findViewById(R.id.btn_visa_checkout);

// Visa Checkout functionality is supported for devices running on KITKAT or above.
// Below version check needs to be added for client apps supporting minSdkVersion = 16
if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
  visaCheckoutButton.init(this, profile, purchaseInfo, new VisaCheckoutSdk.VisaCheckoutResultListener() {
                @Override
                public void onButtonClick(LaunchReadyHandler launchReadyHandler) {
                    launchReadyHandler.launch();
                }

                @Override
                public void onResult(VisaPaymentSummary visaPaymentSummary) {
                    if (VisaPaymentSummary.PAYMENT_SUCCESS.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Success");
                    } else if (VisaPaymentSummary.PAYMENT_CANCEL.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Canceled");
                    } else if (VisaPaymentSummary.PAYMENT_ERROR.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Error");
                    } else if (VisaPaymentSummary.PAYMENT_FAILURE.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Generic Unknown failure");
                    }
                }
            });
}
```
Override the onResult() method as shown above to receive a `VisaPaymentSummary` object from Visa Checkout, which contains the encrypted payload, if requested and available.

## <span style="color:#122d98">Setting Up a custom checkout button</span>
You also have the option to launch the Visa Checkout SDK using your own custom button instead of Visa Checkout button. For instance, you may want to create a custom button and launch the Visa Checkout SDK whenever the button is tapped. Here’s how you can do it:

- Call the `VisaCheckoutSdk.initManualCheckoutSession` method in `onCreate` method of the Activity that initializes the Visa Checkout SDK.

- The `onReady` callback function here returns an object of type `ManualCheckoutLaunchHandler` on which you can execute the `launch()` function later, when the button is tapped. You may create a variable in this Activity as follows to hold on to the launch handler object:

```java
private ManualCheckoutSession.ManualCheckoutLaunchHandler launchHandler;
```
The following code snippet shows how to use the `VisaCheckoutSdk.initManualCheckoutSession` method in `onCreate` of the Activity you present Visa Checkout SDK from:

```java
public class CustomButtonActivity extends AppCompatActivity {
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        Profile profile = new Profile.ProfileBuilder("abc123...", Environment.SANDBOX)
                .setProfileName("default")
                .build();

        PurchaseInfo purchaseInfo = new PurchaseInfo.PurchaseInfoBuilder(new BigDecimal("10.23"),
                PurchaseInfo.Currency.USD)
                .build();

        setUpCustomButton(profile, purchaseInfo);
    }

    setUpCustomButton(Profile profile, PurchaseInfo purchaseInfo) {
        final ImageView customButton = findViewById(R.id.custom_button);
        customButton.setVisibility(View.VISIBLE);
        customButton.setBackground(VisaCheckoutSdk.getCardArt(this.getApplicationContext()));

        // Visa Checkout is supported for devices running Android KITKAT or above.
        // Below version check needs to be added for apps supporting minSdkVersion = 16
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            VisaCheckoutSdk.initManualCheckoutSession(this, profile, purchaseInfo, new ManualCheckoutSession() {
                @Override
                public void onReady(ManualCheckoutLaunchHandler manualCheckoutLaunchHandler) {
                    // store launch handler for later use on click event
                    launchHandler = manualCheckoutLaunchHandler;
                }

                @Override
                public void onResult(VisaPaymentSummary visaPaymentSummary) {
                    //this call invokes the function VisaCheckoutSdk.initManualCheckoutSession and makes sure that you have the right handler
                    if (VisaPaymentSummary.PAYMENT_SUCCESS.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Success");
                    } else if (VisaPaymentSummary.PAYMENT_CANCEL.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Canceled");
                    } else if (VisaPaymentSummary.PAYMENT_ERROR.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Error");
                    } else if (VisaPaymentSummary.PAYMENT_FAILURE.equalsIgnoreCase(visaPaymentSummary.getStatusName())) {
                        Log.d("AppTag", "Generic Unknown failure");
                    }
                    setUpCustomButton(profile, purchaseInfo);
                }
            });
        }
    }

  ```
- Later in your custom button's click listener you execute the `launch()` function on the launchHandler to launch the Visa Checkout SDK:


```java
customButton.setOnClickListener(new View.OnClickListener() {
    @Override public void onClick(View v) {
        if (launchHandler != null) {
            launchHandler.launch();
        }
    }
});
```


<a name="hybrid_usage"></a>
# **<span style="color:#122d98">Usage - Hybrid Apps</span>**

## <span style="color:#122d98">Configure Visa Checkout SDK</span>

1. Call `VisaCheckoutHybridPlugin.configure()` from the `Activity` that contains the `WebView` used to show Visa Checkout.

```java
// Visa Checkout functionality is supported for devices running on KITKAT or above.
// Below version check needs to be added for client apps supporting minSdkVersion = 16
if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
  VisaCheckoutHybridPlugin.configure(webView);
}
```

That's it!
